//--------------------------------------
//
// D3DApp.h 
// Getting D3D App Class providing Framework for a Direct X Application
// Jason Kim August 2009
//
//--------------------------------------

//#Defines
#ifndef D3DAPP_H
#define D3DAPP_H

//Includes
#include "D3DUtil.h"
#include <string>

class D3DApp
{
public:
	//D3DApp Instance Constructor
	D3DApp(HINSTANCE hInstance, std::string winCaption, D3DDEVTYPE devType, DWORD requestedVP);
	
	//Virtual Destructor (derived classes can destruct)
	virtual ~D3DApp();

	//HInstance and HWnd function calls
	HINSTANCE getAppInstance();
	HWND	  getMainWindow();

	// Functions for D3D Framework.  Derived classes can overrides these Functions 
	virtual bool checkDeviceCaps()     
	{ 
		return true; 
	}
	
	virtual void onLostDevice()        
	{
		//

	}
	virtual void onResetDevice()       
	{
		//
	}
	virtual void updateScene(float dt) 
	{
		//
	}
	virtual void drawScene()           
	{
		//
	}

	//Window Creation
	virtual void initMainWindow();
	virtual void initDirect3D();
	virtual int run();
	virtual LRESULT msgProc(UINT msg, WPARAM wParam, LPARAM lParam);

	void enableFullScreenMode(bool enable);
	bool isDeviceLost();

protected:
	//Protected means derived classes have access to modify these members of this class
	//Private means derived classes don't have access, so in this case it is protected so that they can access members
	std::string mMainWindowCaption;
	D3DDEVTYPE  mDevType;
	DWORD       mRequestedVP;
	
	// Application, Windows, and Direct3D data members.
	HINSTANCE             mhAppInstance; //App Instance Handler
	HWND                  mhMainWindow; //Window Handler
	IDirect3D9*           md3dObject; //Object Handler
	bool                  mAppPaused;
	D3DPRESENT_PARAMETERS md3dPP;
};

// Globals for convenient access.
extern D3DApp* gd3dApp;
extern IDirect3DDevice9* gd3dDevice;

#endif // D3DAPP_H