//--------------------------------------
//
// DinoSpaceShooter.cpp
// My Simple 2D Space Shooter Game: DINO SPACE SHOOTER!
// Jason Kim August 2009
//
//--------------------------------------

#include "DinoSpaceShooter.h"
#include "FMODApp.h"

//Win API Main Section (Main App Start)
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE prevInstance,
				   PSTR cmdLine, int showCmd)
{
	// Enable run-time memory check for debug builds.
	#if defined(DEBUG) | defined(_DEBUG)
		_CrtSetDbgFlag( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF );
	#endif

	DinoSpaceShooter app(hInstance, "Dino Space Shooter", D3DDEVTYPE_HAL, D3DCREATE_HARDWARE_VERTEXPROCESSING);
	gd3dApp = &app;

	DirectInput di(DISCL_NONEXCLUSIVE|DISCL_FOREGROUND, DISCL_NONEXCLUSIVE|DISCL_FOREGROUND);
	gDInput = &di;

	return gd3dApp->run();
}

//Constructor for Dino Space Shooter!
DinoSpaceShooter::DinoSpaceShooter(HINSTANCE hInstance, std::string winCaption, D3DDEVTYPE devType, DWORD requestedVP)
: D3DApp(hInstance, winCaption, devType, requestedVP), BULLET_SPEED(2500.0f), MAX_SHIP_SPEED(1500.0f),
	SHIP_ACCEL(1000.0f), SHIP_DRAG(0.85f),ENEMY_SPEED(250.0f)
{
	if(!checkDeviceCaps())
	{
		MessageBox(0, "checkDeviceCaps() Failed", 0, 0);
		PostQuitMessage(0);
	}

	HR(D3DXCreateSprite(gd3dDevice, &mSprite));

	HR(D3DXCreateTextureFromFile(gd3dDevice, "earth.bmp", &mBkgdTex));
	HR(D3DXCreateTextureFromFile(gd3dDevice, "animDino.bmp", &mShipTex));
	HR(D3DXCreateTextureFromFile(gd3dDevice, "fireballz.bmp", &mBulletTex));

	//-----------------------
	//Enemy Texture Setup
	//----------------------
	HR(D3DXCreateTextureFromFile(gd3dDevice, "steggo.bmp", &mEnemy1Tex));
	mEnemy1Center = D3DXVECTOR3(64.0f, 64.0f, 0.0f);

	HR(D3DXCreateTextureFromFile(gd3dDevice, "trirobotops.bmp", &mEnemy2Tex));
	mEnemy2Center = D3DXVECTOR3(64.0f,64.0f,0.0f);

	HR(D3DXCreateTextureFromFile(gd3dDevice, "lazerbeemy.bmp", &mEnemyBulletTex));
	mEnemyBulletCenter = D3DXVECTOR3(32.0f,32.0f,0.0f);
	
	//Setup Enemy Spawn Points
	RECT ScreenRect;
	GetClientRect(mhMainWindow,&ScreenRect);
	SpawnPoint1.x = (float) ScreenRect.left - 400.0f;
	SpawnPoint1.y = (float) ScreenRect.top + 600.0f;
	SpawnPoint1.z = 0.0f;

	SpawnPoint2.x = (float) ScreenRect.left;
	SpawnPoint2.y = (float) ScreenRect.top + 600.0f;
	SpawnPoint2.z = 0.0f;

	SpawnPoint3.x = (float) ScreenRect.left + 400;
	SpawnPoint3.y = (float) ScreenRect.top + 600.0f;
	SpawnPoint3.z = 0.0f;

	bossHealth = 100.0f;
	//-----------------


	//Expolsion Anim
	HR(D3DXCreateTextureFromFile(gd3dDevice, "hearts3.bmp", &mBoomFrame));
	mBoomCenter = D3DXVECTOR3(32.0f, 32.0f, 0.0f);
	//

	//Player Variables
	playerHealth = 100.0f;
	playerScore = 0.0f;
	playerPowerup = false;
	playerFrametime = 0.0f;
	playerCurrentFrame = 0;
	//---------------

	//---- Background and Ship-------------
	mBkgdCenter = D3DXVECTOR3(256.0f, 256.0f, 0.0f);
	mShipCenter = D3DXVECTOR3(64.0f, 64.0f, 0.0f);
	mBulletCenter = D3DXVECTOR3(32.0f, 32.0f, 0.0f);

	mShipPos      = D3DXVECTOR3(0.0f, 0.0f, 0.0f);
	mShipSpeed    = 0.0f;
	mShipRotation = 0.0f;
	//----------------------------

	//----------------------------
	//Font Creation For Title Screen/Game Over/Win Screen
	//----------------------------
	D3DXFONT_DESC fontDesc;
	fontDesc.Height          = 40;
    fontDesc.Width           = 20;
    fontDesc.Weight          = FW_BOLD;
    fontDesc.MipLevels       = 0;
    fontDesc.Italic          = true;
    fontDesc.CharSet         = DEFAULT_CHARSET;
    fontDesc.OutputPrecision = OUT_DEFAULT_PRECIS;
    fontDesc.Quality         = DEFAULT_QUALITY;
    fontDesc.PitchAndFamily  = DEFAULT_PITCH | FF_DONTCARE;
    _tcscpy_s(fontDesc.FaceName, _T("Times New Roman"));

	HR(D3DXCreateFontIndirect(gd3dDevice, &fontDesc, &mFont));
	//----------------------------

	//---------------------------
	//Font Creation For Player Stats
	//---------------------------
	D3DXFONT_DESC fontPlayerDesc;
	fontPlayerDesc.Height          = 25;
    fontPlayerDesc.Width           = 0;
    fontPlayerDesc.Weight          = 0;
    fontPlayerDesc.MipLevels       = 1;
    fontPlayerDesc.Italic          = false;
    fontPlayerDesc.CharSet         = DEFAULT_CHARSET;
    fontPlayerDesc.OutputPrecision = OUT_DEFAULT_PRECIS;
    fontPlayerDesc.Quality         = DEFAULT_QUALITY;
    fontPlayerDesc.PitchAndFamily  = DEFAULT_PITCH | FF_DONTCARE;
    _tcscpy_s(fontPlayerDesc.FaceName, _T("Times New Roman"));

	HR(D3DXCreateFontIndirect(gd3dDevice, &fontPlayerDesc, &mPlayerFont));
	//---------------------------------------------------
	
	//SOUND
	Sound::Instance()->init();//Init FMOD
	Sound::Instance()->load("background.wav",1); //Load Background Song
	Sound::Instance()->load("roar.wav",2); //Explosion Sound
	Sound::Instance()->play(false,1);  //Play Background Music

	onResetDevice();
}

//Deconstructor
DinoSpaceShooter::~DinoSpaceShooter()
{
	
	//Legacy Stuff from old Sprite Demo (STILL NEEDED)
	ReleaseCOM(mSprite);//sprite
	ReleaseCOM(mBkgdTex);//backgrounds
	ReleaseCOM(mShipTex);//player
	ReleaseCOM(mBulletTex);//player bullets

	//Release Enemy
	ReleaseCOM(mEnemy1Tex);//enemy type1 (kamikaze)
	ReleaseCOM(mEnemy2Tex);//enemy type2 (gunner)
	ReleaseCOM(mEnemyBulletTex);//enemy bullets

	ReleaseCOM(mBoomFrame);

	//Release Font
	ReleaseCOM(mFont); // Game Over Screen/Title Screen/Win Screen
	ReleaseCOM(mPlayerFont);//Player Stats Life,Points

	//Release Sounds
	Sound::Instance()->unload(1);
	Sound::Instance()->unload(2);
}

bool DinoSpaceShooter::checkDeviceCaps()
{	
	D3DCAPS9 caps;
	HR(gd3dDevice->GetDeviceCaps(&caps));

	return true;

}

void DinoSpaceShooter::onLostDevice()
{
	HR(mFont->OnLostDevice());
	HR(mPlayerFont->OnLostDevice());

	HR(mSprite->OnLostDevice());
}

void DinoSpaceShooter::onResetDevice()
{

	// Call the onResetDevice of other objects.
	HR(mFont->OnResetDevice());
	HR(mPlayerFont->OnResetDevice());

	HR(mSprite->OnResetDevice());

	// Sets up the camera 1000 units back looking at the origin.
	D3DXMATRIX V;
	D3DXVECTOR3 pos(0.0f, 0.0f, -1000.0f);
	D3DXVECTOR3 up(0.0f, 1.0f, 0.0f);
	D3DXVECTOR3 target(0.0f, 0.0f, 0.0f);
	D3DXMatrixLookAtLH(&V, &pos, &target, &up);
	HR(gd3dDevice->SetTransform(D3DTS_VIEW, &V));

	// The following code defines the volume of space the camera sees.
	D3DXMATRIX P;
	RECT R;
	GetClientRect(mhMainWindow, &R);
	float width  = (float)R.right;
	float height = (float)R.bottom;
	D3DXMatrixPerspectiveFovLH(&P, D3DX_PI*0.25f, width/height, 1.0f, 5000.0f);
	HR(gd3dDevice->SetTransform(D3DTS_PROJECTION, &P));

	// This code sets texture filters, which helps to smooth out distortions
	// when you scale a texture.  
	HR(gd3dDevice->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR));
	HR(gd3dDevice->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR));
	HR(gd3dDevice->SetSamplerState(0, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR));

	// This line of code disables Direct3D lighting.
	HR(gd3dDevice->SetRenderState(D3DRS_LIGHTING, false));
	
	// The following code specifies an alpha test and reference value.
	HR(gd3dDevice->SetRenderState(D3DRS_ALPHAREF, 10));
	HR(gd3dDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATER));

	// The following code is used to setup alpha blending.
	HR(gd3dDevice->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE));
	HR(gd3dDevice->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_SELECTARG1));
	HR(gd3dDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA));
	HR(gd3dDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA));

	// Indicates that we are using 2D texture coordinates.
	HR(gd3dDevice->SetTextureStageState(
		0, D3DTSS_TEXTURETRANSFORMFLAGS, D3DTTFF_COUNT2));

}

void DinoSpaceShooter::updateScene(float dt)
{

	//Keep Track of Game Time
	static float GameTimer = 0.0f;
	GameTimer += dt;

	// Get snapshot of input devices.
	gDInput->poll();


	if(playerHealth <= 0)
	{	
		playerHealth = 100.0f;
		//Dead Player
		GameFlow = 4;	
	}

	if(GameFlow > 0 && GameFlow < 4)
	{
		//game has started
		//every 30 seconds increase level
		if(GameTimer > 5.0f && GameFlow == 1)
		{
			GameFlow = 2 ;
		}
	}

	// Update game objects.
	updateShip(dt);

	//Update Explosions
	updateExplosion(dt);
	//GameFlow 0 - Title Screen
	//GameFlow 1 - Enemies Spawnning
	//GameFlow 2 - Enemies1+2 Spawnning
	//GameFlow 3 - Boss Spawn 
	//GameFlow 4 - Game Over
	//GameFlow 5 - Win Screen
	switch(GameFlow)
	{
	case 0:
		//Title Screen
		//Continual Reset of Game Time to 0
		GameTimer = 0.0f;
		break;
	case 1:
		//Enemy Type 1
		updateEnemy1(dt);
		updateBullets(dt);
		updateEnemyBullet(dt);
		break;
	case 2:
		//Enemy Type 1 + 2
		updateEnemy1(dt);
		updateEnemy2(dt);
		updateEnemyBullet(dt);
		updateBullets(dt);
		break;
	case 3:
		//Boss Spawn....
		updateEnemy1(dt);
		updateEnemy2(dt);
		updateEnemyBullet(dt);
		updateBullets(dt);
		break;
	case 4:
		//Game Over Screen
		//Continual Reset of Game Time to 0
		updateEnemyBullet(dt);
		updateBullets(dt);
		GameTimer = 0.0f;
		break;
	case 5:
		//Win Screen
		//Continual Reset of Game Time to 0 
		GameTimer = 0.0f;
		break;
	case 6:
		//Pause Screen
		break;
	}

}

//Explosion Animation Update
void DinoSpaceShooter::updateExplosion(float dt)
{
	std::list<BoomInfo>::iterator i = mBoomList.begin();
	while ( i != mBoomList.end() )
	{
		//Time
		i->life += dt;
		i->frametime += dt;

		if(i->mCurrentFrame > 29)
		{
			i = mBoomList.erase(i);
				
		}
		else
		{
			//Play Animation at 30FPS
			if(i->frametime >= 1.0f / 30.0f )
			{
			
				i->mCurrentFrame++;
				i->frametime = 0.0f;
				//for looping anim
				/*if(i->mCurrentFrame > 29)
				{
					i->mCurrentFrame = 0;
				}*/	
			}
			++i;
		}
			
	}
	
}

//Enemy Bullet Update loop
void DinoSpaceShooter::updateEnemyBullet(float dt)
{
	// Now loop through each bullet, and update its position.
	std::list<EnemyBulletInfo>::iterator i = mEnemyBulletList.begin();
	while( i != mEnemyBulletList.end() )
	{
		// Accumualte the time the bullet has lived.
		i->life += dt;

		// If the bullet has lived for two seconds, kill it.  By now the
		// bullet should have flown off the screen and cannot be seen.
		if(i->life >= 5.0f)
			i = mEnemyBulletList.erase(i);

		// Otherwise, update its position by moving along its directional
		// path.
		else
		{
			//Check For Bullet Collision With Player

			RECT RPlayer; //Rectangle of player
			RECT REnemyBullet; //Rectangle of Bullet
			RPlayer.left = (long) mShipPos.x;
			RPlayer.right = (long) mShipPos.x+ (long) 128.0f; //(position + width)
			RPlayer.top  = (long) mShipPos.y;
			RPlayer.bottom = (long) mShipPos.y+ (long) 128.0f; //(position + height)

			REnemyBullet.left = (long) i->pos.x;
			REnemyBullet.right = (long) i->pos.x+ (long) 64.0f; //(position + width)
			REnemyBullet.top = (long) i->pos.y;
			REnemyBullet.bottom = (long) i->pos.y+ (long) 64.0f;

			RECT Collide; //Not the most accurate, will implement better in future...
			if(IntersectRect(&Collide,&RPlayer,&REnemyBullet))
			{
				//damage player
				playerHealth -= 10.0f;
				//mark hit
				i->bHit = true;
			}

			if(i->bHit == true)
			{
				i = mEnemyBulletList.erase(i);
			}
			else
			{
				D3DXVECTOR3 dir(-sinf(i->rotation), -cosf(i->rotation), 0.0f);
				i->pos += dir * ENEMY_SPEED * dt;
				++i;

				
			}
	
			
		}

		
	}
}


//-----------------
//ENEMY2 UPDATE LOOP
//-----------------
void DinoSpaceShooter::updateEnemy2(float dt)
{


	if( bEnemy2SpawnFlag == true)
	{
		if(mEnemy2List.size() >= 2)
		{
			bEnemy2SpawnFlag = false;
		}
		else
		{
			//Spawn 2 Enemies
			Enemy2Info Enemy2A; //Spawn an Enemy
			Enemy2Info Enemy2B; //Spawn an Enemy

			Enemy2A.pos = SpawnPoint1;
			Enemy2A.pos.x += 100.0f;
			Enemy2B.pos = SpawnPoint3;
			Enemy2B.pos.x -= 100.0f;
						
			Enemy2A.rotation = mShipRotation;
			Enemy2B.rotation = mShipRotation;

			Enemy2A.enemy2timer = 0.0f;
			Enemy2B.enemy2timer = 0.0f;

			Enemy2A.enemyShotTimer = 0.0f;
			Enemy2B.enemyShotTimer = 0.0f;

			//Enemy just born.
			Enemy2A.life     = 0.0f;
			Enemy2B.life	 = 0.0f;

			Enemy2A.logicflow = 0;
			Enemy2B.logicflow = 0;

			// Add the Enemy to the list.
			mEnemy2List.push_back(Enemy2A);
			mEnemy2List.push_back(Enemy2B);

			bEnemy2SpawnFlag = false;
		}


		
	}

	if(mEnemy2List.size() <= 0)
	{
		bEnemy2SpawnFlag = true;
	}

	std::list<Enemy2Info>::iterator i = mEnemy2List.begin();
	while( i != mEnemy2List.end() )
	{
		i->life += dt;
		i->enemy2timer += dt;
		i->enemyShotTimer += dt;

		//Enemy2 are permanent until destroyed!
		if(i->health <=0)
		{

			//On death Play Explosion
			Sound::Instance()->play(false,2);
			BoomInfo ExplosionA;
			ExplosionA.life = 0.0f;
			ExplosionA.frametime = 0.0f;
			ExplosionA.mCurrentFrame = 0;
			ExplosionA.pos = i->pos;
			ExplosionA.rotation = mShipRotation;
			mBoomList.push_back(ExplosionA);

			playerScore += 500.0f;
			i = mEnemy2List.erase(i);
		}
		else
		{

			//check collision
			//Check Enemy1 Ship Collision With Player
			RECT RPlayer; //Rectangle of player
			RECT REnemy2; //Rectangle of Enemy
			RPlayer.left = (long) mShipPos.x;
			RPlayer.right = (long) mShipPos.x+ (long) 128.0f; //(position + width)
			RPlayer.top  = (long) mShipPos.y;
			RPlayer.bottom = (long) mShipPos.y+ (long) 128.0f; //(position + height)

			REnemy2.left = (long) i->pos.x;
			REnemy2.right = (long) i->pos.x+ (long) 128.0f; //(position + width)
			REnemy2.top = (long) i->pos.y;
			REnemy2.bottom = (long) i->pos.y+ (long) 128.0f;

			RECT Collide;
			if(IntersectRect(&Collide,&RPlayer,&REnemy2))
			{
				//Collided
				//Enemy2 and Player inv to each other
				//Nothing for now?
			}

			//update position
			switch(i->logicflow)
			{
			case 0:
				//Spawnning come down for a few seconds
				if(i->enemy2timer > 1.25f) 
				{
					//change logic flow
					i->logicflow = 1;
					//reset timer
					i->enemy2timer = 0.0f;
				}
				else
				{
					//Come down
					D3DXVECTOR3 dir(-sinf(i->rotation), -cosf(i->rotation), 0.0f);
					i->pos += dir * ENEMY_SPEED * dt;
					
				}
				break;
			case 1:
				//Done Coming Down / Now Move L/R or Shoot

				if(i->enemy2timer > 1.0f)
				{
					int TheRandom;
					TheRandom = rand() % 3;

					switch(TheRandom)
					{
					case 0:
						//Go Shoot From Center
						i->logicflow = 8;
						//reset timer
						i->enemy2timer = 0.0f;
						i->enemyShotTimer = 0.0f;
						break;
					case 1:
						//Go Move Left From Center
						i->logicflow = 2;
						//reset timer
						i->enemy2timer = 0.0f;
						break;
					case 2:
						//Go Move Right From Center
						i->logicflow = 3;
						//reset timer
						i->enemy2timer = 0.0f;
						break;
					}
				}
				
				break;
			case 2:
				//Move Left From Center
				//Spawnning come down for a few seconds
				if(i->enemy2timer > 1.0f) 
				{
					//change logic flow
					i->logicflow = 4;
					//reset timer
					i->enemy2timer = 0.0f;
				}
				else
				{
					//Move Left
					D3DXVECTOR3 dir(90.0f, 0.0, 0.0f);
					i->pos -= dir * 2 * dt;
					
				}
				break;
			case 3:
				//Move Right From Center
				//Spawnning come down for a few seconds
				if(i->enemy2timer > 1.0f) 
				{
					//change logic flow
					i->logicflow = 5;
					//reset timer
					i->enemy2timer = 0.0f;
				}
				else
				{
					//Move Right
					D3DXVECTOR3 dir(90.0f, 0.0, 0.0f);
					i->pos += dir * 2 * dt;
				}
				break;
			case 4:
				//Done Moving Left From Center / Now Move R or Shoot
				if(i->enemy2timer > 1.0f)
				{
					int TheRandom;
					TheRandom = rand() % 2;

					switch(TheRandom)
					{
					case 0:
						//Go Shoot From Left 
						i->logicflow = 9;
						//reset timer
						i->enemy2timer = 0.0f;
						i->enemyShotTimer = 0.0f;
						break;
					case 1:
						//Go Move Right Back to Center
						i->logicflow = 6;
						//reset timer
						i->enemy2timer = 0.0f;
						break;
					}
				}
				break;
			case 5:
				//Done Moving Right From Center / Now Move L or Shoot
				if(i->enemy2timer > 1.0f)
				{
					int TheRandom;
					TheRandom = rand() % 2;

					switch(TheRandom)
					{
					case 0:
						//Go Shoot From Right 
						i->logicflow = 10;
						//reset timer
						i->enemy2timer = 0.0f;
						i->enemyShotTimer = 0.0f;
						break;
					case 1:
						//Go Move Left Back to Center
						i->logicflow = 7;
						//reset timer
						i->enemy2timer = 0.0f;
						break;
					}
				}
				break;
			case 6:
				//Move Right From Left Back to Center
				if(i->enemy2timer > 1.0f) 
				{
					//change logic flow
					i->logicflow = 1;
					//reset timer
					i->enemy2timer = 0.0f;
				}
				else
				{
					//Move Right
					D3DXVECTOR3 dir(90.0f, 0.0, 0.0f);
					i->pos += dir * 2 * dt;
				}
				break;
			case 7:
				//Move Left from Right Back to Center
				if(i->enemy2timer > 1.0f) 
				{
					//change logic flow
					i->logicflow = 1;
					//reset timer
					i->enemy2timer = 0.0f;
				}
				else
				{
					//Move Right
					D3DXVECTOR3 dir(90.0f, 0.0, 0.0f);
					i->pos -= dir * 2 * dt;
				}
				break; 
			case 8:
				//Shoot From Center
				if(i->enemy2timer > 2.0f)
				{
					//Done Shooting
					i->enemy2timer =  0.0f;
					i->enemyShotTimer = 0.0f;
					i->logicflow = 1;

				}
				else
				{
					//Shoot every 0.2 seconds
					if(i->enemyShotTimer > 0.4f)
					{
						EnemyBulletInfo EnemyBullet;
						EnemyBullet.pos = D3DXVECTOR3(i->pos.x, i->pos.y-50.0f, i->pos.z);
						int RandomShot = rand() % 3;
						float RotationShot;
						switch(RandomShot)
						{
						case 0:
							RotationShot = 0.0f;
							break;
						case 1:
							RotationShot = 125.0f;
							break;
						case 2:
							RotationShot = -125.0f;
							break;
						}
						EnemyBullet.rotation = RotationShot; //mShipRotation;
						EnemyBullet.life = 0.0f;
						EnemyBullet.bHit = false;
						mEnemyBulletList.push_back(EnemyBullet);
						i->enemyShotTimer = 0.0f;
					}

				}
				break;
			case 9:
				//Shoot From Left
				if(i->enemy2timer > 2.0f)
				{
					//Done Shooting
					i->enemyShotTimer = 0.0f;
					i->enemy2timer = 0.0f;
					i->logicflow = 4;

				}
				else
				{
					//Shoot
					if(i->enemyShotTimer > 0.4f)
					{
						EnemyBulletInfo EnemyBullet;
						EnemyBullet.pos = D3DXVECTOR3(i->pos.x, i->pos.y-50.0f, i->pos.z);
						int RandomShot = rand() % 3;
						float RotationShot;
						switch(RandomShot)
						{
						case 0:
							RotationShot = 0.0f;
							break;
						case 1:
							RotationShot = 125.0f;
							break;
						case 2:
							RotationShot = -125.0f;
							break;
						}
						EnemyBullet.rotation = RotationShot; // mShipRotation;
						EnemyBullet.life = 0.0f;
						EnemyBullet.bHit = false;
						mEnemyBulletList.push_back(EnemyBullet);
						i->enemyShotTimer = 0.0f;
					}
				}
				break;
			case 10:
				//Shoot From Right
				if(i->enemy2timer > 2.0f)
				{
					//Done Shooting
					i->enemyShotTimer = 0.0f;
					i->enemy2timer = 0.0f;
					i->logicflow = 5;

				}
				else
				{
					//Shoot
					if(i->enemyShotTimer > 0.4f)
					{
						EnemyBulletInfo EnemyBullet;
						EnemyBullet.pos = D3DXVECTOR3(i->pos.x, i->pos.y-50.0f, i->pos.z);
						int RandomShot = rand() % 3;
						float RotationShot;
						switch(RandomShot)
						{
						case 0:
							RotationShot = 0.0f;
							break;
						case 1:
							RotationShot = 125.0f;
							break;
						case 2:
							RotationShot = -125.0f;
							break;
						}
						EnemyBullet.rotation = RotationShot; // mShipRotation;
						EnemyBullet.life = 0.0f;
						EnemyBullet.bHit = false;
						mEnemyBulletList.push_back(EnemyBullet);
						i->enemyShotTimer = 0.0f;
					}
				}
				break;
			}
			++i;

		}
	}

		
}

//-----------------
//ENEMY1 UPDATE LOOP
//-----------------
void DinoSpaceShooter::updateEnemy1(float dt)
{

	static float SpawnTimer = 0.0f;
	SpawnTimer += dt;

	if( bEnemy1SpawnFlag == true )
	{
		//Check Count of Total Ships
		//if 5 enemies on screen, turn off flag
		if(mEnemy1List.size() >= 5)
		{
			bEnemy1SpawnFlag = false;
		}

		//SpawnTimer, Spawn Enemy every 0.5 seconds
		if(SpawnTimer > 0.5f)
		{
			// Enemy Just Spawned, so Reset Timer
			SpawnTimer = 0.0f;

			Enemy1Info Enemy1; //Spawn an Enemy

			int TheRandom = rand() % 3;

			switch(TheRandom)
			{
			case 0:
				Enemy1.pos = SpawnPoint1;
				break;
			case 1:
				Enemy1.pos = SpawnPoint2;
				break;
			case 2:
				Enemy1.pos = SpawnPoint3;
				break;
			}
			
			Enemy1.rotation = mShipRotation;
			//Enemy1 just born.
			Enemy1.life     = 0.0f;

			// Add the Enemy to the list.
			mEnemy1List.push_back(Enemy1);
		}



	}

	// Now loop through each Enemy1, and update its position.
	std::list<Enemy1Info>::iterator i = mEnemy1List.begin();
	if(mEnemy1List.size() <= 0)
	{
		//If No Enemy1 Ships on Screen, Spawn some Enemy1 Ships Every Spawn Timer ticks
		bEnemy1SpawnFlag = true;

	}
	while( i != mEnemy1List.end() )
	{
		// Accumualte the time the bullet has lived.
		i->life += dt;

		// If the Enemy has lived for 5 seconds, kill it.  By now the
		// Enemy should have flown off the screen and cannot be seen.
		// If Enemy Health is lessthanequalto 0 then kill it
		if((i->life >= 5.0f) || (i->health <= 0) )
		{

			if(i->health <= 0 && playerHealth > 0)
			{
				playerScore += 100.0f;
				
				//On death Play Explosion
				Sound::Instance()->play(false,2);
				BoomInfo ExplosionA;
				ExplosionA.life = 0.0f;
				ExplosionA.frametime = 0.0f;
				ExplosionA.mCurrentFrame = 0;
				ExplosionA.pos = i->pos;
				ExplosionA.rotation = mShipRotation;
				mBoomList.push_back(ExplosionA);
			}
			i = mEnemy1List.erase(i);
		}
		else
		{
			// Otherwise, update its position by moving along its directional
			// path.
			
			//Check Enemy1 Ship Collision With Player
			RECT RPlayer; //Rectangle of player
			RECT REnemy1; //Rectangle of Enemy
			RPlayer.left = (long) mShipPos.x;
			RPlayer.right = (long) mShipPos.x+ (long) 128.0f; //(position + width)
			RPlayer.top  = (long) mShipPos.y;
			RPlayer.bottom = (long) mShipPos.y+ (long) 128.0f; //(position + height)

			REnemy1.left = (long) i->pos.x;
			REnemy1.right = (long) i->pos.x+ (long) 128.0f; //(position + width)
			REnemy1.top = (long) i->pos.y;
			REnemy1.bottom = (long) i->pos.y+ (long) 128.0f;

			RECT Collide;
			if(IntersectRect(&Collide,&RPlayer,&REnemy1))
			{
				//Collided
				//Mark Enemy For Deletion (health = 0)
				i->health = -1.0f;
				//Damage Player
				playerHealth -= 10.0f;

			}


			D3DXVECTOR3 dir(-sinf(i->rotation), -cosf(i->rotation), 0.0f);
			i->pos += dir * ENEMY_SPEED * dt;
			++i;


		}

		
	}
	
}

//update player ship
void DinoSpaceShooter::updateShip(float dt)
{
	static float updateDelay = 0.0f;
	updateDelay += dt;
	playerFrametime += dt;

	//Update Player Animation
	if(playerCurrentFrame > 29)
	{
		playerCurrentFrame = 0;
			
	}
	else
	{
		//Play Animation at 30FPS
		if(playerFrametime >= 1.0f / 30.0f )
		{
		
			playerCurrentFrame++;
			playerFrametime = 0.0f;
			//for looping anim
			/*if(i->mCurrentFrame > 29)
			{
				i->mCurrentFrame = 0;
			}*/	
		}
	}



	//Depending on Gameflow
	if(GameFlow == 0)
	{
		//on Title Screen
		//Check Input for Start
		if( gDInput->keyDown(DIK_SPACE) )
		{
			//Title Screen up for half second! (or more later)
			if(updateDelay > 0.5f)
			{
				GameFlow = 1;
				playerHealth = 100.0f;
				playerScore = 0.0f;
			}
		}
	}
	if(GameFlow == 4)
	{
		//Game over Screen
		if( gDInput->keyDown(DIK_SPACE) )
		{
			//Title Screen up for half second! (or more later)
			if(updateDelay > 0.5f)
			{
				GameFlow = 0;
				playerHealth = 100.0f;
				playerScore = 0.0f;

			}
		}
	}
	if(GameFlow > 0 && GameFlow != 4)
	{

		// Clamp top speed.
		if( mShipSpeed > MAX_SHIP_SPEED )	 mShipSpeed =  MAX_SHIP_SPEED;
		if( mShipSpeed < -MAX_SHIP_SPEED )	 mShipSpeed = -MAX_SHIP_SPEED;

		// Rotate counterclockwise when looking down -z axis (i.e., rotate
		// clockwise when looking down the +z axis.
		D3DXVECTOR3 shipDir(-sinf(mShipRotation), cosf(mShipRotation), 0.0f);

		// Update position and speed based on time.
		//mShipPos   += shipDir * mShipSpeed * dt;
		
		//Make Ship Follow Mouse Cursor!
		mShipPos.x = gDInput->mousePosX();
		mShipPos.y = -(gDInput->mousePosY());
		
		mShipSpeed -= SHIP_DRAG * mShipSpeed * dt;

	}
}

//update player bullets
void DinoSpaceShooter::updateBullets(float dt)
{
	// Make static so that its value persists across function calls.
	static float fireDelay = 0.0f;

	// Accumulate time.
	fireDelay += dt;

	// Did the user press the spacebar key and has 0.1 seconds passed?
	// We can only fire one bullet every 0.1 seconds.  If we do not
	// put this delay in, the ship will fire bullets way too fast.
	//gDInput->keyDown(DIK_SPACE) || 
	if( ((gDInput->mouseButtonDown(0)) && fireDelay > 0.15f ) && GameFlow > 0  && GameFlow != 4)
	{
		BulletInfo bullet;

		// Remember the ship is always drawn at the center of the window--
		// the origin.  Therefore, bullets originate from the origin.
		bullet.pos      = D3DXVECTOR3(mShipPos.x,mShipPos.y, mShipPos.z);

		// The bullets rotation should match the ship's rotating at the
		// instant it is fired.  
		bullet.rotation = mShipRotation;

		// Bullet just born.
		bullet.life     = 0.0f;

		// Add the bullet to the list.
		mBulletList.push_back(bullet);

		// A bullet was just fired, so reset the fire delay.
		fireDelay = 0.0f;

	
	}

	// Now loop through each bullet, and update its position.
	std::list<BulletInfo>::iterator i = mBulletList.begin();
	while( i != mBulletList.end() )
	{
		// Accumualte the time the bullet has lived.
		i->life += dt;

		// If the bullet has lived for two seconds, kill it.  By now the
		// bullet should have flown off the screen and cannot be seen.
		if(i->life >= 2.0f)
			i = mBulletList.erase(i);

		// Otherwise, update its position by moving along its directional
		// path.  Code similar to how we move the ship--but no drag.
		else
		{
			//Check For Bullet Collision With Enemies

			//loop through all enemies see if bullet is colliding with them
			std::list<Enemy1Info>::iterator j = mEnemy1List.begin();

			while( j != mEnemy1List.end() )
			{
				RECT RBullet; //Rectangle of Bullet
				RECT REnemy; //Rectangle of Enemy
				RBullet.left = (long) i->pos.x;
				RBullet.right = (long) i->pos.x+ (long) 64.0f; //(position + width)
				RBullet.top  = (long) i->pos.y;
				RBullet.bottom = (long) i->pos.y+ (long) 64.0f; //(position + height)

				REnemy.left = (long) j->pos.x;
				REnemy.right = (long) j->pos.x+ (long) 128.0f; //(position + width)
				REnemy.top = (long) j->pos.y;
				REnemy.bottom = (long) j->pos.y+ (long) 128.0f;

				RECT Collide;
				if(IntersectRect(&Collide,&RBullet,&REnemy))
				{
					//Collided
					j->health -= 1.0f;
					i->bHit = true;
					
				}
				
				++j;

			}

			std::list<Enemy2Info>::iterator k = mEnemy2List.begin();
			while( k != mEnemy2List.end() )
			{
				RECT RBullet; //Rectangle of Bullet
				RECT REnemy; //Rectangle of Enemy
				RBullet.left = (long) i->pos.x;
				RBullet.right = (long) i->pos.x+ (long) 64.0f; //(position + width)
				RBullet.top  = (long) i->pos.y;
				RBullet.bottom = (long) i->pos.y+ (long) 64.0f; //(position + height)

				REnemy.left = (long) k->pos.x;
				REnemy.right = (long) k->pos.x+ (long) 128.0f; //(position + width)
				REnemy.top = (long) k->pos.y;
				REnemy.bottom = (long) k->pos.y+ (long) 128.0f;

				RECT Collide;
				if(IntersectRect(&Collide,&RBullet,&REnemy))
				{
					//Collided
					k->health -= 1.0f;
					i->bHit = true;
					
				}
				
				++k;
				
			}
			
			

			if(i->bHit == true)
			{
				i = mBulletList.erase(i);
			}
			else
			{
				D3DXVECTOR3 dir(-sinf(i->rotation), cosf(i->rotation), 0.0f);
				i->pos += dir * BULLET_SPEED * dt;
				++i;
			}
			
			



		}

		
	}
}

//draw scene
void DinoSpaceShooter::drawScene()
{
	// Clear the backbuffer and depth buffer.
	HR(gd3dDevice->Clear(0, 0, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, 0xffffffff, 1.0f, 0));

	HR(gd3dDevice->BeginScene());
	
	HR(mSprite->Begin(D3DXSPRITE_OBJECTSPACE|D3DXSPRITE_DONOTMODIFY_RENDERSTATE));

	//Always Draw Background
	drawBkgd();
	
	//GameFlow 0 - Title Screen
	//GameFlow 1 - Enemies Spawnning
	//GameFlow 2 - Enemies1+2 Spawnning
	//GameFlow 3 - Boss Spawn 
	//GameFlow 4 - Game Over
	//GameFlow 5 - Win Screen
	switch(GameFlow)
	{
	case 0:
		//Title Screen
		drawTitleScreen();//Draw Title Screen
		//Don't Draw Ship
		//Don't Draw Enemies
		break;
	case 1:
		//Enemies 1 Spawnning
		//Don't Draw Title Screen
		drawShip();	//Draw Ship
		drawEnemy1(); //Draw Enemy1
		drawPlayerStats(); //Draw Player Stats
		break;
	case 2:
		//Enemies 1+2 Spawnning
		drawShip();
		drawEnemy1();
		drawEnemy2();
		drawPlayerStats();
		break;
	case 3:
		//Boss Spawn
		drawBossLife();

		break;
	case 4:
		drawGameOverScreen();
		//Game Over
		break;
	case 5:
		//Win Screen
		break;
	}
	
	drawEnemyBullets();
	drawBullets();
	drawExplosion();
	HR(mSprite->End());
	HR(gd3dDevice->EndScene());

	// Present the backbuffer.
	HR(gd3dDevice->Present(0, 0, 0, 0));

}

//draw explosions
void DinoSpaceShooter::drawExplosion()
{
	std::list<BoomInfo>::iterator i = mBoomList.begin();
	while ( i != mBoomList.end() )
	{
		// Compute rectangle on texture atlas of the current frame
		// we want to use.
		int x = i->mCurrentFrame / 6; // Row
		int y = i->mCurrentFrame % 6; // Column
		RECT R = {y*64, x*64, (y+1)*64, (x+1)*64};

		// Turn on alpha blending.
		HR(gd3dDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, true));


		D3DXMATRIX T;
		D3DXMatrixTranslation(&T, i->pos.x, i->pos.y, i->pos.z);
		HR(mSprite->SetTransform(&T));
		HR(mSprite->Draw(mBoomFrame, &R, &mBoomCenter, 0, D3DCOLOR_XRGB(255, 255, 255)));


		++i;
	}
	// Draw all the bullets at once.
	HR(mSprite->Flush());


	// Turn off alpha blending.
	HR(gd3dDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, false));


	

}

//No Boss Yet....
void DinoSpaceShooter::drawBossLife()
{
	
	//Draw Title With D3D Font

	static char buffer[256];
 
	sprintf_s(buffer, "BOSS LIFE: %.1f\n");
	
	RECT R;
	GetClientRect(mhMainWindow, &R);

	HR(mFont->DrawText(0, buffer, -1, &R, DT_CENTER | DT_NOCLIP, D3DCOLOR_XRGB( 250, 250, 250)));
}

//Draw Game Over Screen
void DinoSpaceShooter::drawGameOverScreen()
{
	//Draw Title With D3D Font

	static char buffer[256];

	sprintf_s(buffer, "GAME OVER\n"
		"Press Spacebar to Restart!\n"
		"High Score = %.1f\n" , playerScore);
	
	RECT R;
	GetClientRect(mhMainWindow, &R);

	HR(mFont->DrawText(0, buffer, -1, &R, DT_CENTER | DT_VCENTER, D3DCOLOR_XRGB( 250, 250, 250)));

}


//Draw Enemy Bullets
void DinoSpaceShooter::drawEnemyBullets()
{

	// Turn on alpha blending.
	HR(gd3dDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, true));

	// For each bullet...
	std::list<EnemyBulletInfo>::iterator i = mEnemyBulletList.begin();
	while( i != mEnemyBulletList.end() )
	{
		// Set its position and orientation.
		D3DXMATRIX T, R;
		D3DXMatrixRotationZ(&R, -(i->rotation));
		D3DXMatrixTranslation(&T, i->pos.x, i->pos.y, i->pos.z);
		HR(mSprite->SetTransform(&(R*T)));

		// Add it to the batch.
		HR(mSprite->Draw(mEnemyBulletTex, 0, &mEnemyBulletCenter, 0, D3DCOLOR_XRGB(255, 255, 255)));
		++i;
	}
	// Draw all the bullets at once.
	HR(mSprite->Flush());

	// Turn off alpha blending.
	HR(gd3dDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, false));
}

//Draw Enemy2 Ships
void DinoSpaceShooter::drawEnemy2()
{
	
	// Turn on alpha blending.
	HR(gd3dDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, true));

	// For each Enemy1...
	std::list<Enemy2Info>::iterator i = mEnemy2List.begin();
	while( i != mEnemy2List.end() )
	{
		// Set its position and orientation.
		D3DXMATRIX T, R;
		D3DXMatrixRotationZ(&R, i->rotation);
		D3DXMatrixTranslation(&T, i->pos.x, i->pos.y, i->pos.z);
		HR(mSprite->SetTransform(&(R*T)));

		// Add it to the batch.
		HR(mSprite->Draw(mEnemy2Tex, 0, &mEnemy1Center, 0, D3DCOLOR_XRGB(255, 255, 255)));
		++i;
	}
	// Draw all the bullets at once.
	HR(mSprite->Flush());

	// Turn off alpha blending.
	HR(gd3dDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, false));

}

//Draw Player Stats
void DinoSpaceShooter::drawPlayerStats()
{
	// Make static so memory is not allocated every frame.
	static char statbuffer[256];

	sprintf_s(statbuffer, "HEALTH = %.1f\n"
		"SCORE = %.1f\n"
		, playerHealth, playerScore);

	RECT R = {5, 5, 0, 0};
	HR(mPlayerFont->DrawText(0, statbuffer, -1, &R, DT_NOCLIP, D3DCOLOR_XRGB(250,250,250)));
}

//Draw Enemy Ship
void DinoSpaceShooter::drawEnemy1()
{
	// Turn on alpha blending.
	HR(gd3dDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, true));

	// For each Enemy1...
	std::list<Enemy1Info>::iterator i = mEnemy1List.begin();
	while( i != mEnemy1List.end() )
	{
		// Set its position and orientation.
		D3DXMATRIX T, R;
		D3DXMatrixRotationZ(&R, i->rotation);
		D3DXMatrixTranslation(&T, i->pos.x, i->pos.y, i->pos.z);
		HR(mSprite->SetTransform(&(R*T)));

		// Add it to the batch.
		HR(mSprite->Draw(mEnemy1Tex, 0, &mEnemy1Center, 0, D3DCOLOR_XRGB(255, 255, 255)));
		++i;
	}
	// Draw all the bullets at once.
	HR(mSprite->Flush());

	// Turn off alpha blending.
	HR(gd3dDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, false));

}

//Draw Title Screen
void DinoSpaceShooter::drawTitleScreen()
{
	//Draw Title With D3D Font

	static char buffer[256];

	sprintf_s(buffer, "DINO SPACE SHOOTER\n"
		"Press Spacebar to Start!\n"
		"Controls: Shoot/Move with Mouse!\n");
	
	RECT R;
	GetClientRect(mhMainWindow, &R);

	HR(mFont->DrawText(0, buffer, -1, &R, DT_CENTER | DT_NOCLIP, D3DCOLOR_XRGB( 250, 250, 250)));
}

//Draw Background
void DinoSpaceShooter::drawBkgd()
{
	// Set a texture coordinate scaling transform.  Here we scale the texture 
	// coordinates by 5 in each dimension.  This tiles the texture 
	// 5 times over the sprite surface.

	//however now with modified code we are only looking at the background center(earth)
	D3DXMATRIX texScaling;
	D3DXMatrixScaling(&texScaling, 5.0f, 5.0f, 5.0f);
	HR(gd3dDevice->SetTransform(D3DTS_TEXTURE0, &texScaling));

	// Position and size the background sprite

	//Old Way to Move
	//D3DXMATRIX T, S;
	//D3DXMatrixTranslation(&T, -mShipPos.x, -mShipPos.y, -mShipPos.z);
	//D3DXMatrixScaling(&S, 20.0f, 20.0f, 0.0f);
	//HR(mSprite->SetTransform(&(S*T)));
	
	D3DXMATRIX S;
	D3DXMatrixScaling(&S, 20.0f, 20.0f, 0.0f);
	HR(mSprite->SetTransform(&S));

	// Draw the background sprite.
	HR(mSprite->Draw(mBkgdTex, 0, &mBkgdCenter, 0, D3DCOLOR_XRGB(255, 255, 255)));
	HR(mSprite->Flush());

	// Restore defaults texture coordinate scaling transform.
	D3DXMatrixScaling(&texScaling, 1.0f, -1.0f, 0.0f);
	HR(gd3dDevice->SetTransform(D3DTS_TEXTURE0, &texScaling));
}

void DinoSpaceShooter::drawShip()
{
	// Turn on the alpha test.
	HR(gd3dDevice->SetRenderState(D3DRS_ALPHATESTENABLE, true));

	
	// Compute rectangle on texture atlas of the current frame
	// we want to use.
	int x = playerCurrentFrame / 4; // Row
	int y = playerCurrentFrame / 4; // Column
	RECT myR = {y*128, x*128, (y+1)*128, (x+1)*128};

	// Set ships orientation.
	D3DXMATRIX R;
	D3DXMatrixRotationZ(&R, mShipRotation);
	// don't rotate ship
	//HR(mSprite->SetTransform(&R));


	//Translate Ship to Be Where Mouse Cursor is
	D3DXMATRIX T;
	D3DXMatrixTranslation(&T, mShipPos.x,mShipPos.y, 0);
	//HR(mSprite->SetTransform(&T));
	HR(mSprite->SetTransform(&(R*T)));


	// Draw the ship.
	HR(mSprite->Draw(mShipTex, &myR, &mShipCenter, 0, D3DCOLOR_XRGB(255, 255, 255)));
	HR(mSprite->Flush());

	// Turn off the alpha test.
	HR(gd3dDevice->SetRenderState(D3DRS_ALPHATESTENABLE, false));
}

void DinoSpaceShooter::drawBullets()
{
	// Turn on alpha blending.
	HR(gd3dDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, true));

	// For each bullet...
	std::list<BulletInfo>::iterator i = mBulletList.begin();
	while( i != mBulletList.end() )
	{
		// Set its position and orientation.
		D3DXMATRIX T, R;
		D3DXMatrixRotationZ(&R, i->rotation);
		D3DXMatrixTranslation(&T, i->pos.x, i->pos.y+90.0f, i->pos.z);
		HR(mSprite->SetTransform(&(R*T)));

		// Add it to the batch.
		HR(mSprite->Draw(mBulletTex, 0, &mBulletCenter, 0, D3DCOLOR_XRGB(255, 255, 255)));
		++i;
	}
	// Draw all the bullets at once.
	HR(mSprite->Flush());

	// Turn off alpha blending.
	HR(gd3dDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, false));
}