//--------------------------------------
//
// DinoSpaceShooter.h
// My Simple 2D Space Shooter Game: DINO SPACE SHOOTER!
// Jason Kim August 2009
//
//--------------------------------------


#include "D3DApp.h"
#include "DirectInput.h"
#include <crtdbg.h>
#include <list>
#include <tchar.h>
#include <vector>
#include <ctime>



//Player Bullet
struct BulletInfo
{
	BulletInfo();
	~BulletInfo();
	D3DXVECTOR3 pos;
	float rotation;
	float life;

	//Collision Info
	bool bHit;
};

BulletInfo::BulletInfo()
{
	bHit = false;
}
BulletInfo::~BulletInfo()
{
}

//Enemy1 Struct
//Enemy1 Is a Data Struct that houses Enemy of Type 1
//Enemy Tyep1 is a Kamikaze Enemy, dies in 3 shots, tries to collide with player
struct Enemy1Info
{
	Enemy1Info();
	~Enemy1Info();
	D3DXVECTOR3 pos;
	float rotation;
	float life;
	float health;

};

Enemy1Info::Enemy1Info()
{
	health = 3.0f; //3 shots to kill Weak Enemy
}
Enemy1Info::~Enemy1Info()
{
}

//Enemy2 Struct 
//Enemy2 is a Struct for Enemy Type 2
//Enemy Type2 is an Enemy that comes in and shoots Enemy Bullets at the Player
struct Enemy2Info
{
	Enemy2Info();
	~Enemy2Info();
	D3DXVECTOR3 pos;
	float rotation;
	float life;
	float health;
	int logicflow;
	float enemy2timer;
	float enemyShotTimer;
};
Enemy2Info::Enemy2Info()
{
	health = 20.0f; // 20 shots to kill Medium Enemy
	logicflow = 0;
	enemy2timer = 0;
	enemyShotTimer = 0;
}
Enemy2Info::~Enemy2Info()
{
}
//Enemy Bullet
struct EnemyBulletInfo
{
	EnemyBulletInfo();
	~EnemyBulletInfo();
	D3DXVECTOR3 pos;
	float rotation;
	float life;

	//Collision Info
	bool bHit;
};

EnemyBulletInfo::EnemyBulletInfo()
{
	bHit = false;
}
EnemyBulletInfo::~EnemyBulletInfo()
{
}

//Enemy Boss class (however no boss yet...)
struct TheBoss
{
	TheBoss();
	~TheBoss();
	D3DXVECTOR3 pos;
	float rotation;
	float life;
	float health;
	int logicflow;
	float Bosstimer;
	float BossShotTimer;
};
TheBoss::TheBoss()
{
	
	health = 100.0f; //100 shots to kill boss
	logicflow = 0;
}
TheBoss::~TheBoss()
{
	//Destructor
}

//Explosion Page Flip Animation
struct BoomInfo
{
	BoomInfo();
	~BoomInfo();
	D3DXVECTOR3 pos;
	float life;
	float frametime;
	int mCurrentFrame;
	float rotation;

};

BoomInfo::BoomInfo()
{
}
BoomInfo::~BoomInfo()
{
}


//Globals for My Simple 2D Shooter Game
//-------------------------------------
//GameFlow 0 - Title Screen
//GameFlow 1 - Enemies Spawnning
//GameFlow 2 - Enemies1+2 Spawnning
//GameFlow 3 - Boss Spawn 
//GameFlow 4 - Game Over
//GameFlow 5 - Win Screen
static int GameFlow = 0;


static bool bEnemy1SpawnFlag = false;
static bool bEnemy2SpawnFlag = false;

//player and main game class.
class DinoSpaceShooter : public D3DApp
{
public:
	DinoSpaceShooter(HINSTANCE hInstance, std::string winCaption, D3DDEVTYPE devType, DWORD requestedVP);
	~DinoSpaceShooter();

	bool checkDeviceCaps();
	void onLostDevice();
	void onResetDevice();
	void updateScene(float dt);
	void drawScene();

	// Helper functions.
	void updateShip(float dt);
	void updateBullets(float dt);
	void drawBkgd();
	void drawShip();
	void drawBullets();

	//Title Screen
	void drawTitleScreen();

	//Game Over Screen
	void drawGameOverScreen();
	
	//Win Screen
	void drawWinScreen();

	//Player Stats
	void drawPlayerStats();

	//Enemy Functions
	void updateEnemy1(float dt);
	void drawEnemy1();
	void updateEnemy2(float dt);
	void drawEnemy2();
	void drawEnemyBullets();
	void updateEnemyBullet(float dt);
	void drawBossLife();
	
	//Enemy Spawn Points
	D3DXVECTOR3 SpawnPoint1;
	D3DXVECTOR3 SpawnPoint2;
	D3DXVECTOR3 SpawnPoint3;

	//Animated Explosion
	void updateExplosion(float dt);
	void drawExplosion();


private:
	
	//sprites
	ID3DXSprite* mSprite;

	//---Font for Title Screen/GameOver/WinScreen--
	ID3DXFont* mFont;
	//--------------------------

	//---Font for Player Stats---
	ID3DXFont* mPlayerFont;
	//---------------------------

	//---Enemies Setup---------------
	IDirect3DTexture9* mEnemy1Tex;
	D3DXVECTOR3 mEnemy1Center;

	const float ENEMY_SPEED;
	std::list<Enemy1Info> mEnemy1List;

	IDirect3DTexture9* mEnemy2Tex;
	D3DXVECTOR3 mEnemy2Center;
	std::list<Enemy2Info> mEnemy2List;

	IDirect3DTexture9* mEnemyBulletTex;
	D3DXVECTOR3 mEnemyBulletCenter;
	std::list<EnemyBulletInfo> mEnemyBulletList;
	float bossHealth;
	//-------------------------

	//Explosion

	std::list<BoomInfo> mBoomList;
	IDirect3DTexture9* mBoomFrame;
	D3DXVECTOR3 mBoomCenter;
	//-------

	//---player varialbes--------
	float playerHealth;
	float playerScore;
	bool playerPowerup;
	float playerFrametime; //Player Animation Frame Time
	int playerCurrentFrame;
	//---------------------------

	//backgroundtexture
	IDirect3DTexture9* mBkgdTex;
	D3DXVECTOR3 mBkgdCenter;

	//ship texture
	IDirect3DTexture9* mShipTex;
	D3DXVECTOR3 mShipCenter;
	D3DXVECTOR3 mShipPos;
	float       mShipSpeed;
	float       mShipRotation;

	IDirect3DTexture9* mBulletTex;
	D3DXVECTOR3 mBulletCenter;
	std::list<BulletInfo> mBulletList;

	const float BULLET_SPEED;
	const float MAX_SHIP_SPEED;
	const float SHIP_ACCEL;
	const float SHIP_DRAG; 
};
