//--------------------------------------
//
// DirectInput.cpp 
// Recieves Dinput callback access to Keyboard and Mouse 
// Jason Kim August 2009
//
//--------------------------------------

//Includes
#include "D3DUtil.h"
#include "DirectInput.h"
#include "D3DApp.h"

//Direct Input Pointer Device
DirectInput* gDInput = 0;

//Constructor
DirectInput::DirectInput(DWORD keyboardCoopFlags, DWORD mouseCoopFlags)
{

	//Clear Keyboard and Mouse old States
	ZeroMemory(mKeyboardState, sizeof(mKeyboardState));
	ZeroMemory(&mMouseState, sizeof(mMouseState));

	//Windows Result Get App Instance for Keyboard/MouseInput
	HR(DirectInput8Create(gd3dApp->getAppInstance(), DIRECTINPUT_VERSION, 
		IID_IDirectInput8, (void**)&mDInput, 0));

	//Get Keyboard Devices for App Window
	HR(mDInput->CreateDevice(GUID_SysKeyboard, &mKeyboard, 0));
	HR(mKeyboard->SetDataFormat(&c_dfDIKeyboard));
	HR(mKeyboard->SetCooperativeLevel(gd3dApp->getMainWindow(), keyboardCoopFlags));
	HR(mKeyboard->Acquire());

	//Get Mouse Devices for App Window
	HR(mDInput->CreateDevice(GUID_SysMouse, &mMouse, 0));
	HR(mMouse->SetDataFormat(&c_dfDIMouse2));
	HR(mMouse->SetCooperativeLevel(gd3dApp->getMainWindow(), mouseCoopFlags));
	HR(mMouse->Acquire());
}

//Deconstructor
DirectInput::~DirectInput()
{
	//Unaacuire all Devices, free upstuff
	ReleaseCOM(mDInput);
	mKeyboard->Unacquire();
	mMouse->Unacquire();
	ReleaseCOM(mKeyboard);
	ReleaseCOM(mMouse);
}

//Poll Devices
void DirectInput::poll()
{
	// Poll keyboard.
	HRESULT hr = mKeyboard->GetDeviceState(sizeof(mKeyboardState), (void**)&mKeyboardState); 
	if( FAILED(hr) )
	{
		// Keyboard lost, zero out keyboard data structure.
		ZeroMemory(mKeyboardState, sizeof(mKeyboardState));

		 // Try to acquire for next time we poll.
		hr = mKeyboard->Acquire();
	}

	// Poll mouse.
	hr = mMouse->GetDeviceState(sizeof(DIMOUSESTATE2), (void**)&mMouseState); 
	if( FAILED(hr) )
	{
		// Mouse lost, zero out mouse data structure.
		ZeroMemory(&mMouseState, sizeof(mMouseState));

		// Try to acquire for next time we poll.
		hr = mMouse->Acquire(); 
	}
}

bool DirectInput::keyDown(char key)
{
	return (mKeyboardState[key] & 0x80) != 0;
}

bool DirectInput::mouseButtonDown(int button)
{
	return (mMouseState.rgbButtons[button] & 0x80) != 0;
}

float DirectInput::mouseDX()
{
	return (float)mMouseState.lX;
}

float DirectInput::mouseDY()
{
	return (float)mMouseState.lY;
}

float DirectInput::mouseDZ()
{
	return (float)mMouseState.lZ;
}

//Added to Get Windows Mouse Position
//Not 100% accurate Mouse Coordinates, but meh its my first try huh? not too bad for my first try IMO
float DirectInput::mousePosX()
{

	//Need to get half of X window Screen Size and Subtract
	RECT MyRect;
	GetClientRect(gd3dApp->getMainWindow(),&MyRect);
	float HalfX = (float)MyRect.right/2;

	GetCursorPos(&mMousePoint);
	ScreenToClient(gd3dApp->getMainWindow(),&mMousePoint);
	return (float)mMousePoint.x - HalfX; //Half of 800x600 Resolution
}

float DirectInput::mousePosY()
{
	//Need to get half of Y window Screen Size and Subtract
	RECT MyRect;
	GetClientRect(gd3dApp->getMainWindow(),&MyRect); //use GetClientRect instead of GetWindowRect, WindowRect is whole Windows, Client Rect is for App
	float HalfY = (float)MyRect.bottom/2;

	GetCursorPos(&mMousePoint);
	ScreenToClient(gd3dApp->getMainWindow(),&mMousePoint);
	return (float)mMousePoint.y-HalfY; //Half of 800x600 Resolution
}