//Jason Kim
//I have chosen to include FMOD as the sound system for this simple 2D shooter game, to show I can implement 3rd Party Software into modules.
//FMOD is a free Sound System API for the PC available at www.fmod.org
//Also made it a Singleton so only one Sound Class Instance is in the entire Scope of Project.

#include "FMODApp.h"

//Private Class Vars
bool Sound::SoundOK = true; //SoundOK to Play Sound
char * Sound::currentSound; //Filename

//FMOD API
FMOD_RESULT Sound::result;
FMOD_SYSTEM * Sound::fmodsystem;
FMOD_SOUND * Sound::sound1;
FMOD_SOUND * Sound::sound2;
FMOD_CHANNEL * Sound::channel;

//Singleton Instance
Sound* Sound::m_pInstance; //global static pointer used to insure single instance of class

//function to get single instance of Sound (called by Sound::Instance()->etc.)
Sound* Sound::Instance()
{
	if(!m_pInstance)
	{
		m_pInstance = new Sound;
	}
	return m_pInstance;
}

//Deconstructor
Sound::~Sound()
{
	if(m_pInstance != 0)
	{
		//delete one sound instance
		delete m_pInstance;
	}
}


//Init FMOD sound
void Sound::init (void) 
{
    //create the sound system
    result = FMOD_System_Create(&fmodsystem);
    if (result != FMOD_OK) 
		SoundOK = false;
    
	//if SoundOK init system. If fails, sound not OK
    if (SoundOK) 
		result = FMOD_System_Init(fmodsystem,2, FMOD_INIT_NORMAL, 0);
    
	if (result != FMOD_OK) 
		SoundOK = false;
    
	//sets initial sound volume (mute)
    if (SoundOK) 
		FMOD_Channel_SetVolume(channel,0.0f);
}


//loads a sound file
void Sound::load (const char * filename,int SoundID) 
{
    currentSound = (char *)filename;

    if (SoundOK) 
	{
		switch(SoundID)
		{
		case 1:
			result = FMOD_Sound_Release(sound1);
			result = FMOD_System_CreateStream(fmodsystem,currentSound, FMOD_SOFTWARE, 0, &sound1);
			if (result != FMOD_OK) 
				SoundOK = false;
			break;
		case 2:
			result = FMOD_Sound_Release(sound2);
			result = FMOD_System_CreateStream(fmodsystem,currentSound, FMOD_SOFTWARE, 0, &sound2);
			if (result != FMOD_OK) 
				SoundOK = false;
			break;
		}
        
    }
}

//unload sound file
void Sound::unload (int SoundID) 
{

	switch(SoundID)
	{
	case 1:
		if (SoundOK) 
		{
			result = FMOD_Sound_Release(sound1);
		}
		break;
	case 2:
		if (SoundOK) 
		{
			result = FMOD_Sound_Release(sound2);
		}
		break;
	}
    
}

//plays a sound (no argument to leave pause as dafault)
void Sound::play (bool pause, int SoundID) 
{
	switch(SoundID)
	{
	case 1:
		if (SoundOK) 
		{
			//music is looping
			result = FMOD_System_PlaySound(fmodsystem,FMOD_CHANNEL_FREE, sound1, pause, &channel);
			FMOD_Channel_SetMode(channel,FMOD_LOOP_NORMAL);
		}
		break;
	case 2:
		if (SoundOK) 
		{
			//explosion non looping sound
			result = FMOD_System_PlaySound(fmodsystem,FMOD_CHANNEL_FREE, sound2, pause, &channel);
			FMOD_Channel_SetMode(channel,FMOD_LOOP_OFF);
		}
		break;
	}
    
}